/*
 * SaveBuddy
 *
 * Copyright (C) 2000 Mohammad A. Haque <mhaque@haque.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307,
 * USA
 */

#define GAIM_PLUGINS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include "gaim.h"

void *handle;

char *getSaveFileName(char *s) {
	char *screenName = current_user->username;
	int i;
	
	for(i = 0; i < strlen(screenName); i++)
		screenName[i] = toupper(screenName[i]);
	
	s[0] = '\0';
	strcat(s,getenv("HOME"));
	strcat(s,"/.gaim/");
	strcat(s,screenName);
	strcat(s,".savebuddy");
}

/* based on do_export in src/dialogs.c */
void savebuddy(void) {
	FILE *file;
	char *saveFile = g_malloc(BUF_LEN);
	char *buf = g_malloc(BUF_LONG);	
	
	getSaveFileName(saveFile);
	
	toc_build_config(buf,BUF_LONG - 1);
	
	if(strlen(buf) > 15)
		if((file = fopen(saveFile,"w"))) {
			fprintf(file,"%s\n",buf);
			fclose(file);
			chmod(buf,S_IRUSR | S_IWUSR);
		} else {
			g_snprintf(buf,BUF_LONG / 2,"Error writing file %s",saveFile);
			do_error_dialog(buf,"Error");
		}
}

/* restore patch from Kevin Smith <kevin@surfari.com> */
void loadbuddy(void) {
	GList *grp, *grp2;
	char *buf = g_malloc(BUF_LONG);
	char *buf2;
	char *first = g_malloc(64);
	char *file = g_malloc(BUF_LEN);
	FILE *f;
	
	if(g_list_length(groups) > 0)
		return;
	
	getSaveFileName(file);
	
	if(!(f = fopen(file,"r"))) {
		g_snprintf(buf, BUF_LONG / 2,"Error reading file %s",file);
		do_error_dialog(buf,"Error");
		g_free(buf);
		g_free(first);
		return;
	}
	
	fgets(first,64,f);
	
	if(!strcasecmp(first,"Config {\n")) {
		g_free(buf);
		g_free(first);
		return;
	} else if(buf[0] == 'm') {
		buf2 = buf;
		buf = g_malloc(1025);
		g_snprintf(buf,1024,"toc_set_config {%s}\n",buf2);
		g_free(buf2);
	}
	
	fseek(f,0,SEEK_SET);
	
	fread(buf,BUF_LONG,1,f);
	
	grp = groups;
	
	while(grp) {
		grp2 = grp->next;
		remove_group((struct group *)grp->data);
		grp = grp2;
	}
	
	parse_toc_buddy_list(buf);
	
	serv_save_config();
	
	build_edit_tree();
	build_permit_tree();
	
	g_free(buf);
	g_free(first);
}

void gaim_plugin_init(void *h) {
	handle = h;
	
	gaim_signal_connect(handle,event_signon,loadbuddy,NULL);
	gaim_signal_connect(handle,event_signoff,savebuddy,NULL);
}

char *name() {
	return "SaveBuddy Version 0.2\n";
}

char *description() {
	return "Saves a local copy of your buddylist to ~/.gaim/SCREENNAME.savebuddy when you signoff and restores from your saved copy if the buddylist on the server has been purged.";
}
